/*****************************************************************************
 *
 * FILE:	wa.h
 * DESCRIPTION:	Wyvern: associated array library header file
 * DATE:	Thu, Oct 21 2004
 * UPDATED:	Thu, Mar 30 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2004-2006 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2004-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: wa.h,v 1.7 2006/03/30 04:47:26 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_WA_H
#define	_WA_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

/******************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	WAL_VERSION_NUMBER	0x10101010L
#define	WAL_VERSION		"WAL/1.1.1"
#define	WAL_VERSION_TEXT	WAL_VERSION " (2006/03/30)"
#define	WAL_VERSION_TEXT_LONG \
	"Wyvern Associated Array Library 1.1.1, Thu, Mar 30 2006"

/*****************************************************************************/

#define	WA_DEFAULT_SIZE	(0)
typedef struct _AAValue {
  const char *		name;
  struct _AAValue *	next;
} AAValue;

typedef struct _AAHash {
  const char *		key;
  AAValue *		value;
  unsigned int		count;	/* number of values */
  struct _AAHash *	next;
} AAHash;
typedef AAHash *	AArray;

/*****************************************************************************
 *
 *	Global function declaration
 *
 *****************************************************************************/
extern AArray *		wa_new(size_t);
extern void		wa_free(AArray *);
extern const char *	wa_put(AArray *, const char *, const char *);
extern const char *	wa_get(AArray *, const char *);
extern AAValue *	wa_put_plus(AArray *, const char *, const char *);
extern AAValue *	wa_get_plus(AArray *, const char *);
extern char *		wa_get_all(AArray *, const char *, const char *);
extern int		wa_delete(AArray *, const char *);
extern int		wa_flush(AArray *);
extern int		wa_foreach(AArray *,
				   void (*)(const char *, const char *));

#endif	/* _WA_H */
